;**********************************************************************
;   PROGRAMME DE CLIGNOTEMENT D'UNE LED CONNECTEE SUR LE PORTA.2      *
;   D'UNE PIC16F84. PROGRAMME D'ENTRAINEMENT AU FONCTIONNEMENT        *
;   DES PICS.                                                         *  
;                                                                     *
;**********************************************************************
;                                                                     *
;    NOM:      LED-CLI                                                *
;    Date:     09/02/2001                                             *
;    Version:  1.0                                                    *
;    Circuit:  Platine d'essais                                       *
;    Auteur:   Bigonoff                                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Fichier requis: P16F84.inc                                       *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes: Ce petit programme permet de faire clignoter une LED      *
;           sur le port A.2  une frquence de 1Hz                    *
;           Ce programme fait partie de la leon 6 des cours          *
;                                                                     *
;**********************************************************************


	LIST      p=16F84            ; Dfinition de processeur
	#include <p16F84.inc>        ; Dfinitions de variables

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _HS_OSC

; '__CONFIG' prcise les paramtres encods dans le processeur au moment de
; la programmation du processeur. Les dfinitions sont dans le fichier include.
; Voici les valeurs et leurs dfinitions :
;	_CP_ON		Code protection ON : impossible de relire
;	_CP_OFF		Code protection OFF
;	_PWRTE_ON	Timer reset sur power on en service
;	_PWRTE_OFF	Timer reset hors-service
;	_WDT_ON		Watch-dog en service
;	_WDT_OFF	Watch-dog hors service
;	_LP_OSC		Oscillateur quartz basse vitesse
;	_XT_OSC		Oscillateur quartz moyenne vitesse
;	_HS_OSC		Oscillateur quartz grande vitesse
;	_RC_OSC		Oscillateur  rseau RC

;*********************************************************************
;                              ASSIGNATIONS                          *
;*********************************************************************

OPTIONVAL	EQU	H'0008'	; Valeur registre option
                                ; Rsistance pull-up ON
                                ; Pas de prscaler

;*********************************************************************
;                             DEFINE                                 *
;*********************************************************************

#DEFINE LED	PORTA,2		; Led rouge
#DEFINE BOUTON	PORTB,2		; bouton-poussoir


;*********************************************************************
;                             MACRO                                  *
;*********************************************************************

LEDON macro	
		bsf	LED
	endm

LEDOFF macro
		bcf	LED
	endm

;*********************************************************************
;                   DECLARATIONS DE VARIABLES                        *
;*********************************************************************

	CBLOCK 0x00C   		; dbut de la zone variables
	
	cmpt1 : 1		; compteur de boucles 1
	cmpt2 : 1		; compteur de boucles 2
	cmpt3 : 1		; compteur de boucles 3

        ENDC			; Fin de la zone                        

;**********************************************************************
;                      DEMARRAGE SUR RESET                            *
;**********************************************************************

	org 0x000 		; Adresse de dpart aprs reset
  	goto    init		; Adresse 0: initialiser

;*********************************************************************
;                       INITIALISATIONS                              *
;*********************************************************************

init
	clrf	PORTA		; Sorties portA  0
	clrf	PORTB		; sorties portB  0
        bsf	STATUS,RP0	; slectionner banque 1
	clrf	EEADR		; permet de diminuer la consommation
	movlw	OPTIONVAL	; charger masque
	movwf	OPTION_REG	; initialiser registre option

			; Effacer RAM
			; ------------
	movlw	0x0c		; initialisation pointeur
	movwf	FSR		; pointeur d'adressage indirec
init1
	clrf	INDF		; effacer ram
	incf	FSR,f		; pointer sur suivant
	btfss	FSR,6		; tester si fin zone atteinte (>=40)
	goto	init1		; non, boucler
	btfss	FSR,4		; tester si fin zone atteinte (>=50)
	goto	init1		; non, boucler

			; initialisations spcifiques
			; ---------------------------
	bcf	LED		; LED en sortie (banque1)
	bcf	STATUS,RP0	; repasser banque 0
	goto	start		; sauter au programme principal

;*********************************************************************
;                      SOUS-ROUTINE DE TEMPORISATION                 *
;*********************************************************************
;---------------------------------------------------------------------
; Cette sous-routine introduit un retard de 500.000 s.
; Elle ne reoit aucun paramtre et n'en retourne aucun
;---------------------------------------------------------------------
tempo
	movlw	2		; pour 2 boucles
	movwf	cmpt3		; initialiser compteur3
boucle3
	clrf	cmpt2		; effacer compteur2
boucle2
	clrf	cmpt1		; effacer compteur1
boucle1
	nop			; perdre 1 cycle
	decfsz	cmpt1 , f	; dcrmenter compteur1
	goto	boucle1		; si pas 0, boucler	
	decfsz	cmpt2 , f	; si 0, dcrmenter compteur 2
	goto	boucle2		; si cmpt2 pas 0, recommencer boucle1
	decfsz	cmpt3 , f	; si 0, dcrmenter compteur 3
	goto	boucle3		; si cmpt3 pas 0, recommencer boucle2
	return			; retour de la sous-routine

;*********************************************************************
;                      PROGRAMME PRINCIPAL                           *
;*********************************************************************

start
	bsf	LED		; allumer la LED : on aurait pu crire
				; LEDON (voir les macros)
	call 	tempo		; appeler la tempo de 0.5s
	bcf	LED		; teindre LED (LEDOFF)
	call	tempo		; appeler la tempor de 0.5s
	goto start		; boucler


	END 			; directive fin de programme

