;**********************************************************************
;                                                                     *
;    Utilisation du mode sleep avec rveil sur le watchdog            *
;                                                                     *
;**********************************************************************
;                                                                     *
;    NOM:      Sleep                                                  *
;    Date:     22/01/2001                                             *
;    Version:  1.0                                                    *
;    Circuit:  Platine d'essais                                       *
;    Auteur:   Bigonoff                                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Fichier requis: P16F84.inc                                       *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************


	LIST      p=16F84            ; Dfinition de processeur
	#include <p16F84.inc>        ; Dfinitions des constantes

	__CONFIG   _CP_OFF & _WDT_ON & _PWRTE_ON & _HS_OSC

; '__CONFIG' prcise les paramtres encods dans le processeur au moment de
; la programmation du processeur. Les dfinitions sont dans le fichier include.
; Voici les valeurs et leurs dfinitions :
;	_CP_ON		Code protection ON : impossible de relire
;	_CP_OFF		Code protection OFF
;	_PWRTE_ON	Timer reset sur power on en service
;	_PWRTE_OFF	Timer reset hors-service
;	_WDT_ON		Watch-dog en service
;	_WDT_OFF	Watch-dog hors service
;	_LP_OSC		Oscillateur quartz basse vitesse
;	_XT_OSC		Oscillateur quartz moyenne vitesse
;	_HS_OSC		Oscillateur quartz grande vitesse
;	_RC_OSC		Oscillateur  rseau RC

;*********************************************************************
;                              ASSIGNATIONS                          *
;*********************************************************************

OPTIONVAL	EQU	H'008D'	; Valeur registre option
                                ; Prscaler wdt  32

INTERMASK	EQU	H'0090' ; Masque d'interruption
                                ; Interruptions sur RB0 (exemple)


;*********************************************************************
;                             DEFINE                                 *
;*********************************************************************

; exemple
; -------
#DEFINE LED	PORTA,2		; LED de sortie

;*********************************************************************
;                             MACRO                                  *
;*********************************************************************

BANK0	macro
	bcf	STATUS , RP0	; passer banque0
	endm

BANK1	macro
	bsf	STATUS , RP0	; passer banque1
	endm

READEE	macro	adeeprom	; lecture d'une adresse EEPROM
	movlw	adeeprom	; charger adresse eeprom
	movwf	EEADR		; adresse  lire dans registre EEADR
	bsf	STATUS , RP0	; passer en banque1
	bsf	EECON1 , RD	; lancer la lecture EEPROM
	bcf	STATUS , RP0	; repasser en banque0
	movf	EEDATA , w	; charger valeur lue dans W
	endm			; fin de la macro

WRITEE	macro	addwrite	; la donne se trouve dans W
	movwf	EEDATA		; placer data dans registre
	movlw	addwrite	; charger adresse d'criture
	movwf	EEADR		; placer dans registre
	bcf	INTCON , GIE	; interdire interruptions
	bsf	STATUS , RP0	; passer en banque1
	bcf	EECON1 , EEIF	; effacer flag de fin d'criture
	bsf	EECON1 , WREN	; autoriser accs criture
	movlw	0x55		; charger 0x55
	movwf	EECON2		; envoyer commande
	movlw	0xAA		; charger 0xAA
	movwf	EECON2		; envoyer commande
	bsf	EECON1 , WR	; lancer cycle d'criture
	bcf	EECON1 , WREN	; verrouiller prochaine criture
	bsf	INTCON , GIE	; rautoriser interruptions
	bcf	STATUS , RP0	; repasser en banque0
	endm

;*********************************************************************
;                   DECLARATIONS DE VARIABLES                        *
;*********************************************************************

	CBLOCK 0x00C   		; dbut de la zone variables
	w_temp :1		; Sauvegarde du registre W
	status_temp : 1		; Sauvegarde du registre STATUS
        ENDC			; Fin de la zone                        

;*********************************************************************
;                   DECLARATIONS DE LA ZONE EEPROM                   *
;*********************************************************************
	org 0x2100		; adresse dbut zone eeprom
;	DE	0x07		; exemple de data

;#DEFINE	eereload	0x00	; adresse eeprom de la data

;**********************************************************************
;                      DEMARRAGE SUR RESET                            *
;**********************************************************************

	org 0x000 		; Adresse de dpart aprs reset
  	goto    init		; Adresse 0: initialiser

;*********************************************************************
;                       INITIALISATIONS                              *
;*********************************************************************
init
	clrf	PORTA		; Sorties portA  0
	clrf	PORTB		; sorties portB  0
	BANK1			; passer banque1
	clrf	EEADR		; permet de diminuer la consommation
	movlw	OPTIONVAL	; charger masque
	movwf	OPTION_REG	; initialiser registre option

			; Effacer RAM
			; ------------
	movlw	0x0c		; initialisation pointeur
	movwf	FSR		; pointeur d'adressage indirect
init1
	clrf	INDF		; effacer ram
	incf	FSR,f		; pointer sur suivant
	btfss	FSR,6		; tester si fin zone atteinte (>=40)
	goto	init1		; non, boucler
	btfss	FSR,4		; tester si fin zone atteinte (>=50)
	goto	init1		; non, boucler

	bcf	LED		; LED en sortie
	BANK0			; passer banque0

	movlw	INTERMASK	; masque interruption
	movwf	INTCON		; charger interrupt control
	goto	start		; sauter programme principal



;*********************************************************************
;                      PROGRAMME PRINCIPAL                           *
;*********************************************************************

start
	bsf	LED		; Allumage de la LED
	sleep			; mise en sommeil
	bcf	LED		; extinction de la LED
	sleep			; mise en sommeil
	goto start		; boucler


	END 			; directive fin de programme

