;**********************************************************************
;   Ralisation d'un tlrupteur avec un bouton-poussoir sur RB0      *
;   et une LED sur RA2.                                               *
;                                                                     *  
;   Anti-rebond de 260ms avec un quartz de 4MHz                       *
;                                                                     *
;**********************************************************************
;                                                                     *
;    NOM: Tlrupteur                                                 *
;    Date: 17/02/2001                                                 *
;    Version: 1.0                                                     *
;    Circuit: platine d'essais                                        *
;    Auteur: Bigonoff                                                 *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Fichier requis: P16F84.inc                                       *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes: excercice sur les interruptions                           *
;                                                                     *
;**********************************************************************


	LIST      p=16F84            ; Dfinition de processeur
	#include <p16F84.inc>        ; Dfinitions des constantes

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _HS_OSC

; '__CONFIG' prcise les paramtres encods dans le processeur au moment de
; la programmation du processeur. Les dfinitions sont dans le fichier include.
; Voici les valeurs et leurs dfinitions :
;	_CP_ON		Code protection ON : impossible de relire
;	_CP_OFF		Code protection OFF
;	_PWRTE_ON	Timer reset sur power on en service
;	_PWRTE_OFF	Timer reset hors-service
;	_WDT_ON		Watch-dog en service
;	_WDT_OFF	Watch-dog hors service
;	_LP_OSC		Oscillateur quartz basse vitesse
;	_XT_OSC		Oscillateur quartz moyenne vitesse
;	_HS_OSC		Oscillateur quartz grande vitesse
;	_RC_OSC		Oscillateur  rseau RC

;*********************************************************************
;                              ASSIGNATIONS                          *
;*********************************************************************

OPTIONVAL	EQU	B'00000111' ; Valeur registre option
                                    ; Rsistance pull-up ON
                                    ; Interrupt flanc descendant RB0
                                    ; Prscaler timer  256

INTERMASK	EQU	H'0090' ; Masque d'interruption
                                ; Interruptions sur RB0
TIMEBASE	EQU	0x04	; nombre de passages dans tmr0

;*********************************************************************
;                             DEFINE                                 *
;*********************************************************************

; exemples
; ---------
#DEFINE LED	PORTA,2		; LED de sortie
#DEFINE BP	PORTB,0		; bouton-poussoir

;*********************************************************************
;                             MACRO                                  *
;*********************************************************************

BANK0	macro
	bcf	STATUS , RP0	; passer banque0
	endm

BANK1	macro
	bsf	STATUS , RP0	; passer banque1
	endm

;*********************************************************************
;                   DECLARATIONS DE VARIABLES                        *
;*********************************************************************

	CBLOCK 0x00C   		; dbut de la zone variables
	w_temp :1		; Sauvegarde du registre W
	status_temp : 1		; Sauvegarde du registre STATUS
	cmpt : 1		; compteur de passages
        ENDC			; Fin de la zone                        

;**********************************************************************
;                      DEMARRAGE SUR RESET                            *
;**********************************************************************

	org 0x000 		; Adresse de dpart aprs reset
  	goto    init		; Adresse 0: initialiser

;**********************************************************************
;                     ROUTINE INTERRUPTION                            *
;**********************************************************************

			;sauvegarder registres	
			;---------------------
	org 0x004		; adresse d'interruption
	movwf   w_temp  	; sauver registre W
	swapf	STATUS,w	; swap status avec rsultat dans w
	movwf	status_temp	; sauver status swapp

		; switch vers diffrentes interrupts
		;----------------------------------------
	
	btfsc	INTCON,T0IE	; tester si interrupt timer autorise
	btfss	INTCON,T0IF	; oui, tester si interrupt timer en cours
	goto 	intsw1		; non test suivant
	call	inttimer	; oui, traiter interrupt timer
	bcf	INTCON,T0IF	; effacer flag interrupt timer
	goto	restorereg	; et fin d'interruption

intsw1
	call	intrb0		; traiter interrupt RB0
				; si pas timer, forcment RB0
	bcf	INTCON,INTF	; effacer flag interupt RB0
	
			;restaurer registres
			;-------------------
restorereg
	swapf	status_temp,w	; swap ancien status, rsultat dans w
	movwf   STATUS		; restaurer status
	swapf   w_temp,f	; Inversion L et H de l'ancien W
                       		; sans modifier Z
	swapf   w_temp,w  	; Rinversion de L et H dans W
				; W restaur sans modifier status
	retfie  		; return from interrupt

;**********************************************************************
;                     INTERRUPTION TIMER 0                            *
;**********************************************************************
inttimer
	decfsz	cmpt , f	; dcrmenter compteur de passages
	return			; pas 0, fin
	movlw	TIMEBASE	; recharger base de temps
	movwf	cmpt		; recharger compteur de passages
	movlw	B'00010000'	; prparation valeur INTCON
	movwf	INTCON		; interruptions tmr coupes
				; interruptions RB0/INT en service
				; FLAG RB0 EFFACE
				; le Flag GIE sera remis par retfie
	return			; fin d'interruption timer

;**********************************************************************
;                     INTERRUPTION RB0/INT                            *
;**********************************************************************
intrb0
	movlw	b'00000100'	; prparer pour inversion bit
	BANK0			; passer banque0
	xorwf	PORTA , f	; inverser LED
	clrf	TMR0		; reset du timer0
	movlw	B'00100000'	; prparation pour INTCON
	movwf	INTCON		; interruptions INTF hors service
				; interruptions tmr0 en service
				; flag INTF remis  0
	return			; fin d'interruption RB0/INT

;*********************************************************************
;                       INITIALISATIONS                              *
;*********************************************************************

init
	clrf	PORTA		; Sorties portA  0
	clrf	PORTB		; sorties portB  0
	BANK1			; passer banque1
	clrf	EEADR		; permet de diminuer la consommation
	movlw	OPTIONVAL	; charger masque
	movwf	OPTION_REG	; initialiser registre option

			; Effacer RAM
			; ------------
	movlw	0x0c		; initialisation pointeur
	movwf	FSR		; pointeur d'adressage indirect
init1
	clrf	INDF		; effacer ram
	incf	FSR,f		; pointer sur suivant
	btfss	FSR,6		; tester si fin zone atteinte (>=40)
	goto	init1		; non, boucler
	btfss	FSR,4		; tester si fin zone atteinte (>=50)
	goto	init1		; non, boucler


	movlw	TIMEBASE	; charger nbre de passages
	movwf	cmpt		; charger compteur de passages
	bcf	LED		; LED en sortie
	BANK0			; passer banque0
	movlw	INTERMASK	; masque interruption
	movwf	INTCON		; charger interrupt control
	goto	start		; sauter programme principal


;*********************************************************************
;                      PROGRAMME PRINCIPAL                           *
;*********************************************************************

start

	goto start		; boucler





	END 			; directive fin de programme

