;**********************************************************************
;   MESURE DU TEMPS DU TIMER DU WATCHDOG                              *
;   ON REGLE LE PRESCALER SUR WATCHDOG ET AVEC /128                   *
;   DIRECTEMENT APRES POWER ON, ON RESETTE LE WATCHDOG                *
;   ON ALLUME UNE LED 250ms APRES POWER ON                            *
;   ON BOUCLE ET ON ATTEND LE REDEMARRAGE                             *
;   LE TEMPS ENTRE 2 EXTINCTIONS DONNE PRESCALER*128                  *
;                                                                     *
;**********************************************************************
;                                                                     *
;    NOM:      WDT                                                    *
;    Date:     09/02/2001                                             *
;    Version:  1.0                                                    *
;    Circuit:  Platine d'essais                                       *
;    Auteur:   Bigonoff                                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Fichier requis: P16F84.inc                                       *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes: permet de mesurer le temps rel du watchdog               *
;                                                                     *
;**********************************************************************


	LIST      p=16F84            ; Dfinition de processeur
	#include <p16F84.inc>        ; Dfinitions de variables

	__CONFIG   _CP_OFF & _WDT_ON & _PWRTE_ON & _HS_OSC

; '__CONFIG' prcise les paramtres encods dans le processeur au moment de
; la programmation du processeur. Les dfinitions sont dans le fichier include.
; Voici les valeurs et leurs dfinitions :
;	_CP_ON		Code protection ON : impossible de relire
;	_CP_OFF		Code protection OFF
;	_PWRTE_ON	Timer reset sur power on en service
;	_PWRTE_OFF	Timer reset hors-service
;	_WDT_ON		Watch-dog en service
;	_WDT_OFF	Watch-dog hors service
;	_LP_OSC		Oscillateur quartz basse vitesse
;	_XT_OSC		Oscillateur quartz moyenne vitesse
;	_HS_OSC		Oscillateur quartz grande vitesse
;	_RC_OSC		Oscillateur  rseau RC

;*********************************************************************
;                              ASSIGNATIONS                          *
;*********************************************************************

OPTIONVAL	EQU	B'10001111'	; Valeur registre option
                                	; Rsistance pull-up OFF
                                	; prscaler Wdt = 128

;*********************************************************************
;                             DEFINE                                 *
;*********************************************************************

#DEFINE LED	PORTA,2		; Led rouge


;*********************************************************************
;                   DECLARATIONS DE VARIABLES                        *
;*********************************************************************

	CBLOCK 0x00C   		; dbut de la zone variables
	
	cmpt1 : 1		; compteur de boucles 1
	cmpt2 : 1		; compteur de boucles 2

        ENDC			; Fin de la zone                        

;**********************************************************************
;                      DEMARRAGE SUR RESET                            *
;**********************************************************************

	org 0x000 		; Adresse de dpart aprs reset
  	goto    init		; Adresse 0: initialiser

;*********************************************************************
;                       INITIALISATIONS                              *
;*********************************************************************

init
	clrwdt			; effacer watchdog
	clrf	PORTA		; Sorties portA  0
	clrf	PORTB		; sorties portB  0
        bsf	STATUS,RP0	; slectionner banque 1
	clrf	EEADR		; permet de diminuer la consommation
	movlw	OPTIONVAL	; charger masque
	movwf	OPTION_REG	; initialiser registre option

			; Effacer RAM
			; ------------
	movlw	0x0c		; initialisation pointeur
	movwf	FSR		; pointeur d'adressage indirec
init1
	clrf	INDF		; effacer ram
	incf	FSR,f		; pointer sur suivant
	btfss	FSR,6		; tester si fin zone atteinte (>=40)
	goto	init1		; non, boucler
	btfss	FSR,4		; tester si fin zone atteinte (>=50)
	goto	init1		; non, boucler

			; initialisations spcifiques
			; ---------------------------
	bcf	LED		; LED en sortie (banque1)
	bcf	STATUS,RP0	; repasser banque 0
	goto	start		; sauter au programme principal

;*********************************************************************
;                      SOUS-ROUTINE DE TEMPORISATION                 *
;*********************************************************************
;---------------------------------------------------------------------
; Cette sous-routine introduit un retard de 250ms.
; Elle ne reoit aucun paramtre et n'en retourne aucun
;---------------------------------------------------------------------
tempo
	clrf	cmpt2		; effacer compteur2
boucle2
	clrf	cmpt1		; effacer compteur1
boucle1
	nop			; perdre 1 cycle
	decfsz	cmpt1 , f	; dcrmenter compteur1
	goto	boucle1		; si pas 0, boucler	
	decfsz	cmpt2 , f	; si 0, dcrmenter compteur 2
	goto	boucle2		; si cmpt2 pas 0, recommencer boucle1
	return			; retour de la sous-routine

;*********************************************************************
;                      PROGRAMME PRINCIPAL                           *
;*********************************************************************

start
	call	tempo		; on attends 250ms
	bsf	LED		; allumer la LED
loop
	goto	loop		; on reste ici

	END 			; directive fin de programme

