;**********************************************************************
;   Ce fichier est la base de dpart pour une programmation avec      *
;   le PIC 16F84. Il contient  les informations de  base pour         *
;   dmarrer.                                                         *  
;                                                                     *
;   Si les interruptions ne sont pas utilises, supprimez les lignes  *
;   entre ORG 0x004 et l'tiquette init. De plus, les variables       *
;   w_temp et status_temp peuvent tre supprimes.                    *
;                                                                     *
;**********************************************************************
;                                                                     *
;    NOM:                                                             *
;    Date:                                                            *
;    Version:                                                         *
;    Circuit:
;    Auteur:
;                                                                     *
;**********************************************************************
;                                                                     *
;    Fichier requis: P16F84.inc                                       *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************


	LIST      p=16F84            ; Dfinition de processeur
	#include <p16F84.inc>        ; Dfinitions des constantes

	__CONFIG   _CP_OFF & _WDT_ON & _PWRTE_ON & _HS_OSC

; '__CONFIG' prcise les paramtres encods dans le processeur au moment de
; la programmation du processeur. Les dfinitions sont dans le fichier include.
; Voici les valeurs et leurs dfinitions :
;	_CP_ON		Code protection ON : impossible de relire
;	_CP_OFF		Code protection OFF
;	_PWRTE_ON	Timer reset sur power on en service
;	_PWRTE_OFF	Timer reset hors-service
;	_WDT_ON		Watch-dog en service
;	_WDT_OFF	Watch-dog hors service
;	_LP_OSC		Oscillateur quartz basse vitesse
;	_XT_OSC		Oscillateur quartz moyenne vitesse
;	_HS_OSC		Oscillateur quartz grande vitesse
;	_RC_OSC		Oscillateur  rseau RC

;*********************************************************************
;                              ASSIGNATIONS                          *
;*********************************************************************

OPTIONVAL	EQU	H'0040'	; Valeur registre option
                                ; Rsistance pull-up ON
                                ; Interrupt flanc montant RB0
                                ; Prscaler timer  2 (exemple)

INTERMASK	EQU	H'0090' ; Masque d'interruption
                                ; Interruptions sur RB0 (exemple)


;*********************************************************************
;                             DEFINE                                 *
;*********************************************************************

; exemples
; ---------
#DEFINE LED1	PORTB,1		; LED de sortie 1
#DEFINE LEDR	PORTB,2		; LED de tmoin de rception
#DEFINE INPUT	PORTB,0		; entre RS485

;*********************************************************************
;                             MACRO                                  *
;*********************************************************************

;Exemple
;-------
LIREIN macro	
		comf PORTB,0
		andlw	1
	endm
;

;*********************************************************************
;                   DECLARATIONS DE VARIABLES                        *
;*********************************************************************

;exemples
;---------
	CBLOCK 0x00C   		; dbut de la zone variables
	w_temp :1		; Zone de 1 byte
	status_temp : 1		; zone de 1 byte
        ENDC			; Fin de la zone                        

;**********************************************************************
;                      DEMARRAGE SUR RESET                            *
;**********************************************************************

	org 0x000 		; Adresse de dpart aprs reset
  	goto    init		; Adresse 0: initialiser

;**********************************************************************
;                     ROUTINE INTERRUPTION                            *
;**********************************************************************

			;sauvegarder registres	
			;---------------------
	org 0x004		; adresse d'interruption
	movwf   w_temp  	; sauver registre W
	swapf	STATUS,w	; swap status avec rsultat dans w
	movwf	status_temp	; sauver status swapp

			; switch vers diffrentes interrupts
			; inverser ordre pour modifier priorits
			;----------------------------------------
	
	btfsc	INTCON,T0IE	; tester si interrupt timer autorise
	btfss	INTCON,T0IF	; oui, tester si interrupt timer en cours
	goto 	intsw1		; non test suivant
	call	inttimer	; oui, traiter interrupt timer
	bcf	INTCON,T0IF	; effacer flag interrupt timer
	goto	restorereg	; et fin d'interruption
				; SUPPRIMER CETTE LIGNE POUR
				; TRAITER PLUSIEURS INTERRUPT
				; EN 1 SEULE FOIS

intsw1
	btfsc	INTCON,INTE	; tester si interrupt RB0 autorise
	btfss	INTCON,INTF	; oui, tester si interrupt RB0 en cours
	goto 	intsw2		; non sauter au test suivant
	call	intrb0		; oui, traiter interrupt RB0
	bcf	INTCON,INTF	; effacer flag interupt RB0
	goto	restorereg	; et fin d'interruption
				; SUPPRIMER CETTE LIGNE POUR
				; TRAITER PLUSIEURS INTERRUPT
				; EN 1 SEULE FOIS

intsw2
	btfsc	INTCON,RBIE	; tester si interrupt RB4/7 autorise
	btfss	INTCON,RBIF	; oui, tester si interrupt RB4/7 en cours
	goto 	intsw3		; non sauter
	call	intrb4		; oui, traiter interrupt RB4/7
	bcf	INTCON,RBIF	; effacer flag interupt RB4/7
	goto	restorereg	; et fin d'interrupt

intsw3

			;restaurer registres
			;-------------------
restorereg
	swapf	status_temp,w	; swap ancien status, rsultat dans w
	movwf   STATUS		; restaurer status
	swapf   w_temp,f	; Inversion L et H de l'ancien W
                       		; sans modifier Z
	swapf   w_temp,w  	; Rinversion de L et H dans W
				; W restaur sans modifier status
	retfie  		; return from interrupt

;**********************************************************************
;                     INTERRUPTION TIMER 0                            *
;**********************************************************************
inttimer
	return			; fin d'interruption timer
				; peut tre remplac par 
				; retlw pour retour code d'erreur

;**********************************************************************
;                     INTERRUPTION RB0/INT                            *
;**********************************************************************
intrb0
	return			; fin d'interruption RB0/INT
				; peut tre remplac par 
				; retlw pour retour code d'erreur

;**********************************************************************
;                     INTERRUPTION RB0/RB4                            *
;**********************************************************************
intrb4
	return			; fin d'interruption RB0/RB4
				; peut tre remplac par 
				; retlw pour retour code d'erreur


;*********************************************************************
;                       INITIALISATIONS                              *
;*********************************************************************

init
	clrf	PORTA		; Sorties portA  0
	clrf	PORTB		; sorties portB  0
        bsf	STATUS,RP0	; slectionner banque 1
	clrf	EEADR		; permet de diminuer la consommation
	movlw	OPTIONVAL	; charger masque
	movwf	OPTION_REG	; initialiser registre option

			; Effacer RAM
			; ------------
	movlw	0x0c		; initialisation pointeur
	movwf	FSR		; pointeur d'adressage indirec
init1
	clrf	INDF		; effacer ram
	incf	FSR,f		; pointer sur suivant
	btfss	FSR,6		; tester si fin zone atteinte (>=40)
	goto	init1		; non, boucler
	btfss	FSR,4		; tester si fin zone atteinte (>=50)
	goto	init1		; non, boucler



	bcf	TRISA,0		; Bit PORTA.0 en sortie (exemple)

	bcf	STATUS,RP0	; Slectionner banque 0
	movlw	INTERMASK	; masque interruption
	movwf	INTCON		; charger interrupt control


;*********************************************************************
;                      PROGRAMME PRINCIPAL                           *
;*********************************************************************

start

	clrwdt			; effacer watch dog
	goto start		; boucler





	END 			; directive fin de programme

