;**********************************************************************
;   Ce fichier est la base de dpart pour une programmation avec      *
;   le PIC 16F84. Il contient  les informations de  base pour         *
;   dmarrer.                                                         *  
;                                                                     *
;   Si les interruptions ne sont pas utilises, supprimez les lignes  *
;   entre ORG 0x004 et l'tiquette init. De plus, les variables       *
;   w_temp et status_temp peuvent tre supprimes.                    *
;                                                                     *
;**********************************************************************
;                                                                     *
;    NOM:                                                             *
;    Date:                                                            *
;    Version:                                                         *
;    Circuit:                                                         *
;    Auteur:                                                          *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Fichier requis: P16F84.inc                                       *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************


	LIST      p=16F84            ; Dfinition de processeur
	#include <p16F84.inc>        ; Dfinitions des constantes

	__CONFIG   _CP_OFF & _WDT_ON & _PWRTE_ON & _HS_OSC

; '__CONFIG' prcise les paramtres encods dans le processeur au moment de
; la programmation du processeur. Les dfinitions sont dans le fichier include.
; Voici les valeurs et leurs dfinitions :
;	_CP_ON		Code protection ON : impossible de relire
;	_CP_OFF		Code protection OFF
;	_PWRTE_ON	Timer reset sur power on en service
;	_PWRTE_OFF	Timer reset hors-service
;	_WDT_ON		Watch-dog en service
;	_WDT_OFF	Watch-dog hors service
;	_LP_OSC		Oscillateur quartz basse vitesse
;	_XT_OSC		Oscillateur quartz moyenne vitesse
;	_HS_OSC		Oscillateur quartz grande vitesse
;	_RC_OSC		Oscillateur  rseau RC

;*********************************************************************
;                              ASSIGNATIONS                          *
;*********************************************************************

OPTIONVAL	EQU	H'0040'	; Valeur registre option
                                ; Rsistance pull-up ON
                                ; Interrupt flanc montant RB0
                                ; Prscaler timer  2 (exemple)

INTERMASK	EQU	H'0090' ; Masque d'interruption
                                ; Interruptions sur RB0 (exemple)


;*********************************************************************
;                             DEFINE                                 *
;*********************************************************************

; exemple
; -------
#DEFINE LED1	PORTB,1		; LED de sortie 1

;*********************************************************************
;                             MACRO                                  *
;*********************************************************************

BANK0	macro
	bcf	STATUS , RP0	; passer banque0
	endm

BANK1	macro
	bsf	STATUS , RP0	; passer banque1
	endm

READEE	macro	adeeprom	; lecture d'une adresse EEPROM
	movlw	adeeprom	; charger adresse eeprom
	movwf	EEADR		; adresse  lire dans registre EEADR
	bsf	STATUS , RP0	; passer en banque1
	bsf	EECON1 , RD	; lancer la lecture EEPROM
	bcf	STATUS , RP0	; repasser en banque0
	movf	EEDATA , w	; charger valeur lue dans W
	endm			; fin de la macro

WRITEE	macro	addwrite	; la donne se trouve dans W
	LOCAL	loop		; tiquette locale
	movwf	EEDATA		; placer data dans registre
	movlw	addwrite	; charger adresse d'criture
	movwf	EEADR		; placer dans registre
loop
	bcf	INTCON , GIE	; interdire interruptions
	btfsc	INTCON , GIE	; tester si GIE bien  0
	goto	loop		; non, recommencer
	bsf	STATUS , RP0	; passer en banque1
	bcf	EECON1 , EEIF	; effacer flag de fin d'criture
	bsf	EECON1 , WREN	; autoriser accs criture
	movlw	0x55		; charger 0x55
	movwf	EECON2		; envoyer commande
	movlw	0xAA		; charger 0xAA
	movwf	EECON2		; envoyer commande
	bsf	EECON1 , WR	; lancer cycle d'criture
	bcf	EECON1 , WREN	; verrouiller prochaine criture
	bsf	INTCON , GIE	; rautoriser interruptions
	bcf	STATUS , RP0	; repasser en banque0
	endm

;*********************************************************************
;                   DECLARATIONS DE VARIABLES                        *
;*********************************************************************

	CBLOCK 0x00C   		; dbut de la zone variables
	w_temp :1		; Sauvegarde du registre W
	status_temp : 1		; Sauvegarde du registre STATUS
        ENDC			; Fin de la zone                        

;*********************************************************************
;                   DECLARATIONS DE LA ZONE EEPROM                   *
;*********************************************************************
	org 0x2100		; adresse dbut zone eeprom
;	DE	0x07		; exemple de data

;#DEFINE	eereload	0x00	; adresse eeprom de la data

;**********************************************************************
;                      DEMARRAGE SUR RESET                            *
;**********************************************************************

	org 0x000 		; Adresse de dpart aprs reset
  	goto    init		; Adresse 0: initialiser

;**********************************************************************
;                     ROUTINE INTERRUPTION                            *
;**********************************************************************

			;sauvegarder registres	
			;---------------------
	org 0x004		; adresse d'interruption
	movwf   w_temp  	; sauver registre W
	swapf	STATUS,w	; swap status avec rsultat dans w
	movwf	status_temp	; sauver status swapp

			; switch vers diffrentes interrupts
			; inverser ordre pour modifier priorits
			;----------------------------------------
	
	btfsc	INTCON,T0IE	; tester si interrupt timer autorise
	btfss	INTCON,T0IF	; oui, tester si interrupt timer en cours
	goto 	intsw1		; non test suivant
	call	inttimer	; oui, traiter interrupt timer
	bcf	INTCON,T0IF	; effacer flag interrupt timer
	goto	restorereg	; et fin d'interruption
				; SUPPRIMER CETTE LIGNE POUR
				; TRAITER PLUSIEURS INTERRUPT
				; EN 1 SEULE FOIS

intsw1
	btfsc	INTCON,INTE	; tester si interrupt RB0 autorise
	btfss	INTCON,INTF	; oui, tester si interrupt RB0 en cours
	goto 	intsw2		; non sauter au test suivant
	call	intrb0		; oui, traiter interrupt RB0
	bcf	INTCON,INTF	; effacer flag interupt RB0
	goto	restorereg	; et fin d'interruption
				; SUPPRIMER CETTE LIGNE POUR
				; TRAITER PLUSIEURS INTERRUPT
				; EN 1 SEULE FOIS

intsw2
	btfsc	INTCON,RBIE	; tester si interrupt RB4/7 autorise
	btfss	INTCON,RBIF	; oui, tester si interrupt RB4/7 en cours
	goto 	intsw3		; non sauter
	call	intrb4		; oui, traiter interrupt RB4/7
	bcf	INTCON,RBIF	; effacer flag interupt RB4/7
	goto	restorereg	; et fin d'interrupt
				; SUPPRIMER CETTE LIGNE POUR
				; TRAITER PLUSIEURS INTERRUPT
				; EN 1 SEULE FOIS

intsw3
	BANK1			; passer banque1
	btfsc	INTCON,EEIE	; tester si interrupt EEPROM autorise
	btfss	EECON1,EEIF	; oui,tester si interrupt EEPROM
	goto 	restorereg	; non sauter
	call	inteep		; traiter interruption eeprom

	
			;restaurer registres
			;-------------------
restorereg
	swapf	status_temp,w	; swap ancien status, rsultat dans w
	movwf   STATUS		; restaurer status
	swapf   w_temp,f	; Inversion L et H de l'ancien W
                       		; sans modifier Z
	swapf   w_temp,w  	; Rinversion de L et H dans W
				; W restaur sans modifier status
	retfie  		; return from interrupt

;**********************************************************************
;                     INTERRUPTION TIMER 0                            *
;**********************************************************************
inttimer
	return			; fin d'interruption timer
				; peut tre remplac par 
				; retlw pour retour code d'erreur

;**********************************************************************
;                     INTERRUPTION RB0/INT                            *
;**********************************************************************
intrb0
	return			; fin d'interruption RB0/INT
				; peut tre remplac par 
				; retlw pour retour code d'erreur

;**********************************************************************
;                     INTERRUPTION RB0/RB4                            *
;**********************************************************************
intrb4
	return			; fin d'interruption RB0/RB4
				; peut tre remplac par 
				; retlw pour retour code d'erreur

;**********************************************************************
;                     INTERRUPTION EEPROM                             *
;**********************************************************************
inteep
	return			; fin d'interruption eeprom
				; peut tre remplac par 
				; retlw pour retour code d'erreur


;*********************************************************************
;                       INITIALISATIONS                              *
;*********************************************************************

init
	clrf	PORTA		; Sorties portA  0
	clrf	PORTB		; sorties portB  0
	BANK1			; passer banque1
	movlw	OPTIONVAL	; charger masque
	movwf	OPTION_REG	; initialiser registre option
 	clrf	EEADR		; permet de diminuer la consommation

			; Effacer RAM
			; ------------
	movlw	0x0c		; initialisation pointeur
	movwf	FSR		; pointeur d'adressage indirect
init1
	clrf	INDF		; effacer ram
	incf	FSR,f		; pointer sur suivant
	btfss	FSR,6		; tester si fin zone atteinte (>=40)
	goto	init1		; non, boucler
	btfss	FSR,4		; tester si fin zone atteinte (>=50)
	goto	init1		; non, boucler



	bcf	TRISA,0		; Bit PORTA.0 en sortie (exemple)
	BANK0			; passer banque0

	movlw	INTERMASK	; masque interruption
	movwf	INTCON		; charger interrupt control
	goto	start		; sauter programme principal


;*********************************************************************
;                      PROGRAMME PRINCIPAL                           *
;*********************************************************************

start

	clrwdt			; effacer watch dog
	goto start		; boucler





	END 			; directive fin de programme

